--[[
	Name: HideLooted
	Author: Pneumatus
	Description:
		Suppress chat messages when specified items are looted, or items of a lower quality than a specified rarity.
]]--

local HideLooted = LibStub("AceAddon-3.0"):NewAddon("HideLooted", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("HideLooted")

local HL_LOOT_ITEM = LOOT_ITEM:format(".+", ".+item:(%d+).+")
local HL_CREATED_ITEM = CREATED_ITEM:format(".+", ".+item:(%d+).+")

local hex2rarity = {}

StaticPopupDialogs["HL_ERR_ITEMID"] = {
	text = L["Unable to get ItemID for %s.\nTry entering the ItemID directly."],
	button1 = OKAY,
	showAlert = 1,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1,
}

HideLooted.options = {
	type = 'group',
	name = "HideLooted",
	handler = HideLooted,
	args = {
		desc = {
			type = 'description',
			name = L["HL_DESC"],
			order = 100,
		},
		rarity = {
			type = 'select',
			name = L["Minimum Rarity"],
			desc = L["Hide items more common than the specified rarity"],
			get = function(info) return info.handler.db.profile.rarity end,
			set = function(info, val) info.handler.db.profile.rarity = val end,
			values = {},
			order = 110,
		},
		items = {
			type = 'group',
			name = L["Hidden Items"],
			inline = true,
			order = 120,
			args = {
				add = {
					type = 'input',
					name = L["Add"],
					desc = L["Add an item to the Hidden Items list (Valid inputs are: ItemID, ItemName or ItemLink)"],
					get = false,
					set = function(info, val)
						info.handler:AddHiddenItem(val)
					end,
					order = 1,
				},
				del = {
					type = 'input',
					name = L["Delete"],
					desc = L["Remove an item from the Hidden Items list (Valid inputs are: ItemID, ItemName or ItemLink)"],
					get = false,
					set = function(info, val)
						info.handler:DeleteHiddenItem(val)
					end,
					order = 2,
				},
				reset = {
					type = 'execute',
					name = L["Reset"],
					desc = L["Reset Hidden Items list to default"],
					func = function(info)
						for k, _ in pairs(info.handler.db.profile.items) do
							info.handler:DeleteHiddenItem(k)
							if info.handler.defaults.profile.items[k] then
								info.handler:AddHiddenItem(k)
							end
						end
					end,
					width = "full",
					order = -1,
				},
			},
		},
	},
}

HideLooted.defaults = {
	profile = {
		items = {
			["20558"] = true,		-- Warsong Gulch Mark of Honor
			["20559"] = true,		-- Arathi Basin Mark of Honor
			["20560"] = true,		-- Alterac Valley Mark of Honor
			["29024"] = true,		-- Eye of the Storm Mark of Honor
			["29434"] = true,		-- Badge of Justice
			["40752"] = true,		-- Emblem of Heroism
			["40753"] = true,		-- Emblem of Valor
			["42425"] = true,		-- Strand of the Ancients Mark of Honor
			["43228"] = true,		-- Stone Keeper's Shard
			["43589"] = true,		-- Wintergrasp Mark of Honor
			["44990"] = true,		-- Champion's Seal
			["45624"] = true,		-- Emblem of Conquest
			["47241"] = true,		-- Emblem of Triumph
			["47395"] = true,		-- Isle of Conquest Mark of Honor
		},
		rarity = 0,
	},
}

local function FilterLoot(self, event, msg)
	local itemID = msg:match(HL_LOOT_ITEM) or msg:match(HL_CREATED_ITEM)
	if not itemID then return false end
	local itemRarity = hex2rarity[msg:match("(|c%x+)|Hitem")]
	if itemRarity and itemRarity < HideLooted.db.profile.rarity then
		return true
	end
	return HideLooted.db.profile.items[itemID]
end

function HideLooted:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("HideLootedDB", self.defaults)
	
	-- Populate the rarity dropdown for the Options GUI and hex lookup table
	for i = 0, 7 do
		local _, _, _, hex = GetItemQualityColor(i);
		self.options.args.rarity.values[i] = i > 0 and hex.._G["ITEM_QUALITY"..i.."_DESC"].."|r" or "|cffff0000"..L["Show All"].."|r"
		hex2rarity[hex] = i
	end
	
	-- Populate the items list for the Options GUI
	for itemID, show in pairs(self.db.profile.items) do
		if show then
			self:AddHiddenItem(itemID)
		end
	end
	
	-- Set up the InterfaceOptions
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("HideLooted", self.options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("HideLooted", "HideLooted")
	
	local optionsFrame = LibStub("AceConfigDialog-3.0").BlizOptions["HideLooted"].frame
	self:RegisterChatCommand("hidelooted", function() InterfaceOptionsFrame_OpenToCategory(optionsFrame) end)
	self:RegisterChatCommand("hl", function() InterfaceOptionsFrame_OpenToCategory(optionsFrame) end)
end

function HideLooted:OnEnable()
	ChatFrame_AddMessageEventFilter("CHAT_MSG_LOOT", FilterLoot)
end

function HideLooted:OnDisable()
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_LOOT", FilterLoot)
end

function HideLooted:AddHiddenItem(item)
	local itemID = item
	if not itemID:match("^%d+$") then
		local _, itemLink, _, _, _, _, _, _, _, _ = GetItemInfo(item)
		itemID = itemLink and itemLink:match("item:(%d+)")
	end
	
	if itemID then
		self.db.profile.items[itemID] = true
		local _, itemLink, _, _, _, _, _, _, _, _ = GetItemInfo(itemID)
		self.options.args.items.args[itemID] = {
			type = 'description',
			name = ("(%s) %s"):format(itemID, itemLink or "|cffff0000"..L["Unable to retrieve Item Link"].."|r"),
			order = tonumber(itemID),
		}
	else
		StaticPopup_Show("HL_ERR_ITEMID", item)
	end
end

function HideLooted:DeleteHiddenItem(item)
	local itemID = item
	if not itemID:match("^%d+$") then
		local _, itemLink, _, _, _, _, _, _, _, _ = GetItemInfo(item)
		itemID = itemLink and itemLink:match("item:(%d+)")
	end
	
	if itemID then
		self.db.profile.items[itemID] = nil or self.defaults.profile.items[itemID] and false
		self.options.args.items.args[itemID] = nil
	else
		StaticPopup_Show("HL_ERR_ITEMID", item)
	end
end
